@@ -9,16 +9,11 @@
     (at your option) any later version.  See <https://www.gnu.org/licenses/>.
 */
 
+#include <math.h>
 #include "d_mat.h"
 #include "fmpz.h"
 #include "fmpz_mat.h"
 
-#ifdef __GNUC__
-# define sqrt __builtin_sqrt
-#else
-# include <math.h>
-#endif
-
 void
 fmpz_mat_chol_d(d_mat_t R, const fmpz_mat_t A)
 {
